package ru.dbotthepony.mc.otm.container.slotted

import net.minecraft.world.item.ItemStack

fun interface AutomationPlaceItem<in S : ContainerSlot> {
	fun canAutomationPlaceItem(self: S, itemStack: ItemStack): Boolean
}

fun interface AutomationTakeItem<in S : ContainerSlot> {
	fun canAutomationTakeItem(self: S, desired: Int): Boolean
}

fun interface AutomationModifyPlaceCount<in S : ContainerSlot> {
	fun modifyAutomationPlaceCount(self: S, itemStack: ItemStack): Int
}

fun interface AutomationModifyExtractionCount<in S : ContainerSlot> {
	fun modifyAutomationExtractionCount(self: S, desired: Int): Int
}

interface AutomationFilter<in S : ContainerSlot> : AutomationPlaceItem<S>, AutomationTakeItem<S>, AutomationModifyPlaceCount<S>, AutomationModifyExtractionCount<S> {
	override fun modifyAutomationPlaceCount(self: S, itemStack: ItemStack): Int {
		return itemStack.count
	}

	override fun modifyAutomationExtractionCount(self: S, desired: Int): Int {
		return desired
	}
}

fun <S : ContainerSlot, T : S> AutomationFilter<S>.and(other: AutomationFilter<T>): AutomationFilter<T> {
	return object : AutomationFilter<T> {
		override fun canAutomationPlaceItem(self: T, itemStack: ItemStack): Boolean {
			return this@and.canAutomationPlaceItem(self, itemStack) && other.canAutomationPlaceItem(self, itemStack)
		}

		override fun canAutomationTakeItem(self: T, desired: Int): Boolean {
			return this@and.canAutomationTakeItem(self, desired) && other.canAutomationTakeItem(self, desired)
		}

		override fun modifyAutomationPlaceCount(self: T, itemStack: ItemStack): Int {
			return this@and.modifyAutomationPlaceCount(self, itemStack)
		}

		override fun modifyAutomationExtractionCount(self: T, desired: Int): Int {
			return this@and.modifyAutomationExtractionCount(self, desired)
		}
	}
}

